from envi.archs.z80.regs import *
from envi.archs.z80.const import *
# (patrn,mask), mnem, o1type, o1info, o2type, o2info, oplen, immoff, iflags
z80table = [
(('00', 'ff'), 'nop', None, None, None, None, 1, 0, 0),
(('010000', 'ff0000'), 'ld', OPTYPE_Reg, REG_BC, OPTYPE_imm16, None, 3, 1, 0),
(('02', 'ff'), 'ld', OPTYPE_RegMem, REG_BC, OPTYPE_Reg, REG_A, 1, 0, 0),
(('03', 'ff'), 'inc', OPTYPE_Reg, REG_BC, None, None, 1, 0, 0),
(('04', 'ff'), 'inc', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('05', 'ff'), 'dec', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('0600', 'ff00'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_imm8, None, 2, 1, 0),
(('07', 'ff'), 'rlca', None, None, None, None, 1, 0, 0),
(('08', 'ff'), 'ex', OPTYPE_Reg, REG_AF, OPTYPE_RegAlt, REG_AF, 1, 0, 0),
(('09', 'ff'), 'add', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_BC, 1, 0, 0),
(('0a', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_RegMem, REG_BC, 1, 0, 0),
(('0b', 'ff'), 'dec', OPTYPE_Reg, REG_BC, None, None, 1, 0, 0),
(('0c', 'ff'), 'inc', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('0d', 'ff'), 'dec', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('0e00', 'ff00'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_imm8, None, 2, 1, 0),
(('0f', 'ff'), 'rrca', None, None, None, None, 1, 0, 0),
(('1000', 'ff00'), 'djnz', OPTYPE_Ind, None, None, None, 2, 1, 0),
(('110000', 'ff0000'), 'ld', OPTYPE_Reg, REG_DE, OPTYPE_imm16, None, 3, 1, 0),
(('12', 'ff'), 'ld', OPTYPE_RegMem, REG_DE, OPTYPE_Reg, REG_A, 1, 0, 0),
(('13', 'ff'), 'inc', OPTYPE_Reg, REG_DE, None, None, 1, 0, 0),
(('14', 'ff'), 'inc', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('15', 'ff'), 'dec', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('1600', 'ff00'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_imm8, None, 2, 1, 0),
(('17', 'ff'), 'rla', None, None, None, None, 1, 0, 0),
(('1800', 'ff00'), 'jr', OPTYPE_Ind, None, None, None, 2, 1, 0),
(('19', 'ff'), 'add', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_DE, 1, 0, 0),
(('1a', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_RegMem, REG_DE, 1, 0, 0),
(('1b', 'ff'), 'dec', OPTYPE_Reg, REG_DE, None, None, 1, 0, 0),
(('1c', 'ff'), 'inc', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('1d', 'ff'), 'dec', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('1e00', 'ff00'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_imm8, None, 2, 1, 0),
(('1f', 'ff'), 'rra', None, None, None, None, 1, 0, 0),
(('2000', 'ff00'), 'jr', OPTYPE_Cond, COND_NZ, OPTYPE_Ind, None, 2, 1, 0),
(('210000', 'ff0000'), 'ld', OPTYPE_Reg, REG_HL, OPTYPE_imm16, None, 3, 1, 0),
(('220000', 'ff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_HL, 3, 1, 0),
(('23', 'ff'), 'inc', OPTYPE_Reg, REG_HL, None, None, 1, 0, 0),
(('24', 'ff'), 'inc', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('25', 'ff'), 'dec', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('2600', 'ff00'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_imm8, None, 2, 1, 0),
(('27', 'ff'), 'daa', None, None, None, None, 1, 0, 0),
(('2800', 'ff00'), 'jr', OPTYPE_Cond, COND_Z, OPTYPE_Ind, None, 2, 1, 0),
(('29', 'ff'), 'add', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_HL, 1, 0, 0),
(('2a0000', 'ff0000'), 'ld', OPTYPE_Reg, REG_HL, OPTYPE_immmem16, None, 3, 1, 0),
(('2b', 'ff'), 'dec', OPTYPE_Reg, REG_HL, None, None, 1, 0, 0),
(('2c', 'ff'), 'inc', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('2d', 'ff'), 'dec', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('2e00', 'ff00'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_imm8, None, 2, 1, 0),
(('2f', 'ff'), 'cpl', None, None, None, None, 1, 0, 0),
(('3000', 'ff00'), 'jr', OPTYPE_Cond, COND_NC, OPTYPE_Ind, None, 2, 1, 0),
(('310000', 'ff0000'), 'ld', OPTYPE_Reg, REG_SP, OPTYPE_imm16, None, 3, 1, 0),
(('320000', 'ff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_A, 3, 1, 0),
(('33', 'ff'), 'inc', OPTYPE_Reg, REG_SP, None, None, 1, 0, 0),
(('34', 'ff'), 'inc', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('35', 'ff'), 'dec', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('3600', 'ff00'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_imm8, None, 2, 1, 0),
(('37', 'ff'), 'scf', None, None, None, None, 1, 0, 0),
(('3800', 'ff00'), 'jr', OPTYPE_Cond, COND_C, OPTYPE_Ind, None, 2, 1, 0),
(('39', 'ff'), 'add', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_SP, 1, 0, 0),
(('3a0000', 'ff0000'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_immmem16, None, 3, 1, 0),
(('3b', 'ff'), 'dec', OPTYPE_Reg, REG_SP, None, None, 1, 0, 0),
(('3c', 'ff'), 'inc', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('3d', 'ff'), 'dec', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('3e00', 'ff00'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_imm8, None, 2, 1, 0),
(('3f', 'ff'), 'ccf', None, None, None, None, 1, 0, 0),
(('40', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_B, 1, 0, 0),
(('41', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_C, 1, 0, 0),
(('42', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_D, 1, 0, 0),
(('43', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_E, 1, 0, 0),
(('44', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_H, 1, 0, 0),
(('45', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_L, 1, 0, 0),
(('46', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('47', 'ff'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_Reg, REG_A, 1, 0, 0),
(('48', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_B, 1, 0, 0),
(('49', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_C, 1, 0, 0),
(('4a', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_D, 1, 0, 0),
(('4b', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_E, 1, 0, 0),
(('4c', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_H, 1, 0, 0),
(('4d', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_L, 1, 0, 0),
(('4e', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('4f', 'ff'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_Reg, REG_A, 1, 0, 0),
(('50', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_B, 1, 0, 0),
(('51', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_C, 1, 0, 0),
(('52', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_D, 1, 0, 0),
(('53', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_E, 1, 0, 0),
(('54', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_H, 1, 0, 0),
(('55', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_L, 1, 0, 0),
(('56', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('57', 'ff'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_Reg, REG_A, 1, 0, 0),
(('58', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_B, 1, 0, 0),
(('59', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_C, 1, 0, 0),
(('5a', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_D, 1, 0, 0),
(('5b', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_E, 1, 0, 0),
(('5c', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_H, 1, 0, 0),
(('5d', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_L, 1, 0, 0),
(('5e', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('5f', 'ff'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_Reg, REG_A, 1, 0, 0),
(('60', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_B, 1, 0, 0),
(('61', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_C, 1, 0, 0),
(('62', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_D, 1, 0, 0),
(('63', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_E, 1, 0, 0),
(('64', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_H, 1, 0, 0),
(('65', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_L, 1, 0, 0),
(('66', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('67', 'ff'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_Reg, REG_A, 1, 0, 0),
(('68', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_B, 1, 0, 0),
(('69', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_C, 1, 0, 0),
(('6a', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_D, 1, 0, 0),
(('6b', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_E, 1, 0, 0),
(('6c', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_H, 1, 0, 0),
(('6d', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_L, 1, 0, 0),
(('6e', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('6f', 'ff'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_Reg, REG_A, 1, 0, 0),
(('70', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_B, 1, 0, 0),
(('71', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_C, 1, 0, 0),
(('72', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_D, 1, 0, 0),
(('73', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_E, 1, 0, 0),
(('74', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_H, 1, 0, 0),
(('75', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_L, 1, 0, 0),
(('76', 'ff'), 'halt', None, None, None, None, 1, 0, 0),
(('77', 'ff'), 'ld', OPTYPE_RegMem, REG_HL, OPTYPE_Reg, REG_A, 1, 0, 0),
(('78', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_B, 1, 0, 0),
(('79', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_C, 1, 0, 0),
(('7a', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_D, 1, 0, 0),
(('7b', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_E, 1, 0, 0),
(('7c', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_H, 1, 0, 0),
(('7d', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_L, 1, 0, 0),
(('7e', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('7f', 'ff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_A, 1, 0, 0),
(('80', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_B, 1, 0, 0),
(('81', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_C, 1, 0, 0),
(('82', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_D, 1, 0, 0),
(('83', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_E, 1, 0, 0),
(('84', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_H, 1, 0, 0),
(('85', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_L, 1, 0, 0),
(('86', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('87', 'ff'), 'add', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_A, 1, 0, 0),
(('88', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_B, 1, 0, 0),
(('89', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_C, 1, 0, 0),
(('8a', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_D, 1, 0, 0),
(('8b', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_E, 1, 0, 0),
(('8c', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_H, 1, 0, 0),
(('8d', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_L, 1, 0, 0),
(('8e', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_RegMem, REG_HL, 1, 0, 0),
(('8f', 'ff'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_A, 1, 0, 0),
(('90', 'ff'), 'sub', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('91', 'ff'), 'sub', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('92', 'ff'), 'sub', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('93', 'ff'), 'sub', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('94', 'ff'), 'sub', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('95', 'ff'), 'sub', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('96', 'ff'), 'sub', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('97', 'ff'), 'sub', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('98', 'ff'), 'sbc', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('99', 'ff'), 'sbc', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('9a', 'ff'), 'sbc', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('9b', 'ff'), 'sbc', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('9c', 'ff'), 'sbc', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('9d', 'ff'), 'sbc', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('9e', 'ff'), 'sbc', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('9f', 'ff'), 'sbc', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('a0', 'ff'), 'and', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('a1', 'ff'), 'and', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('a2', 'ff'), 'and', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('a3', 'ff'), 'and', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('a4', 'ff'), 'and', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('a5', 'ff'), 'and', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('a6', 'ff'), 'and', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('a7', 'ff'), 'and', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('a8', 'ff'), 'xor', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('a9', 'ff'), 'xor', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('aa', 'ff'), 'xor', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('ab', 'ff'), 'xor', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('ac', 'ff'), 'xor', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('ad', 'ff'), 'xor', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('ae', 'ff'), 'xor', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('af', 'ff'), 'xor', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('b0', 'ff'), 'or', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('b1', 'ff'), 'or', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('b2', 'ff'), 'or', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('b3', 'ff'), 'or', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('b4', 'ff'), 'or', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('b5', 'ff'), 'or', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('b6', 'ff'), 'or', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('b7', 'ff'), 'or', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('b8', 'ff'), 'cp', OPTYPE_Reg, REG_B, None, None, 1, 0, 0),
(('b9', 'ff'), 'cp', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('ba', 'ff'), 'cp', OPTYPE_Reg, REG_D, None, None, 1, 0, 0),
(('bb', 'ff'), 'cp', OPTYPE_Reg, REG_E, None, None, 1, 0, 0),
(('bc', 'ff'), 'cp', OPTYPE_Reg, REG_H, None, None, 1, 0, 0),
(('bd', 'ff'), 'cp', OPTYPE_Reg, REG_L, None, None, 1, 0, 0),
(('be', 'ff'), 'cp', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('bf', 'ff'), 'cp', OPTYPE_Reg, REG_A, None, None, 1, 0, 0),
(('c0', 'ff'), 'ret', OPTYPE_Cond, COND_NZ, None, None, 1, 0, 0),
(('c1', 'ff'), 'pop', OPTYPE_Reg, REG_BC, None, None, 1, 0, 0),
(('c2', 'ff'), 'jp', OPTYPE_Cond, COND_NZ, OPTYPE_Ind, None, 1, 0, 0),
(('c3', 'ff'), 'jp', OPTYPE_imm16, None, None, None, 3, 1, 0),
(('c40000', 'ff0000'), 'call', OPTYPE_Cond, COND_NZ, OPTYPE_imm16, None, 3, 1, 0),
(('c5', 'ff'), 'push', OPTYPE_Reg, REG_BC, None, None, 1, 0, 0),
(('c600', 'ff00'), 'add', OPTYPE_Reg, REG_A, OPTYPE_imm8, None, 2, 1, 0),
(('c7', 'ff'), 'rst', OPTYPE_const, 0, None, None, 1, 0, 0),
(('c8', 'ff'), 'ret', OPTYPE_Cond, COND_Z, None, None, 1, 0, 0),
(('c9', 'ff'), 'ret', None, None, None, None, 1, 0, 0),
(('ca', 'ff'), 'jp', OPTYPE_Cond, COND_Z, OPTYPE_Ind, None, 1, 0, 0),
(('cb00', 'ffff'), 'rlc', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb01', 'ffff'), 'rlc', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb02', 'ffff'), 'rlc', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb03', 'ffff'), 'rlc', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb04', 'ffff'), 'rlc', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb05', 'ffff'), 'rlc', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb06', 'ffff'), 'rlc', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb07', 'ffff'), 'rlc', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb08', 'ffff'), 'rrc', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb09', 'ffff'), 'rrc', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb0a', 'ffff'), 'rrc', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb0b', 'ffff'), 'rrc', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb0c', 'ffff'), 'rrc', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb0d', 'ffff'), 'rrc', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb0e', 'ffff'), 'rrc', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb0f', 'ffff'), 'rrc', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb10', 'ffff'), 'rl', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb11', 'ffff'), 'rl', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb12', 'ffff'), 'rl', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb13', 'ffff'), 'rl', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb14', 'ffff'), 'rl', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb15', 'ffff'), 'rl', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb16', 'ffff'), 'rl', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb17', 'ffff'), 'rl', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb18', 'ffff'), 'rr', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb19', 'ffff'), 'rr', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb1a', 'ffff'), 'rr', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb1b', 'ffff'), 'rr', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb1c', 'ffff'), 'rr', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb1d', 'ffff'), 'rr', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb1e', 'ffff'), 'rr', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb1f', 'ffff'), 'rr', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb20', 'ffff'), 'sla', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb21', 'ffff'), 'sla', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb22', 'ffff'), 'sla', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb23', 'ffff'), 'sla', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb24', 'ffff'), 'sla', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb25', 'ffff'), 'sla', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb26', 'ffff'), 'sla', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb27', 'ffff'), 'sla', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb28', 'ffff'), 'sra', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb29', 'ffff'), 'sra', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb2a', 'ffff'), 'sra', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb2b', 'ffff'), 'sra', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb2c', 'ffff'), 'sra', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb2d', 'ffff'), 'sra', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb2e', 'ffff'), 'sra', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb2f', 'ffff'), 'sra', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb38', 'ffff'), 'srl', OPTYPE_Reg, REG_B, None, None, 2, 0, 0),
(('cb39', 'ffff'), 'srl', OPTYPE_Reg, REG_C, None, None, 2, 0, 0),
(('cb3a', 'ffff'), 'srl', OPTYPE_Reg, REG_D, None, None, 2, 0, 0),
(('cb3b', 'ffff'), 'srl', OPTYPE_Reg, REG_E, None, None, 2, 0, 0),
(('cb3c', 'ffff'), 'srl', OPTYPE_Reg, REG_H, None, None, 2, 0, 0),
(('cb3d', 'ffff'), 'srl', OPTYPE_Reg, REG_L, None, None, 2, 0, 0),
(('cb3e', 'ffff'), 'srl', OPTYPE_RegMem, REG_HL, None, None, 2, 0, 0),
(('cb3f', 'ffff'), 'srl', OPTYPE_Reg, REG_A, None, None, 2, 0, 0),
(('cb40', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb41', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb42', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb43', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb44', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb45', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb46', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb47', 'ffff'), 'bit', OPTYPE_const, 0, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb48', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb49', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb4a', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb4b', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb4c', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb4d', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb4e', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb4f', 'ffff'), 'bit', OPTYPE_const, 1, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb50', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb51', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb52', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb53', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb54', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb55', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb56', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb57', 'ffff'), 'bit', OPTYPE_const, 2, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb58', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb59', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb5a', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb5b', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb5c', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb5d', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb5e', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb5f', 'ffff'), 'bit', OPTYPE_const, 3, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb60', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb61', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb62', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb63', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb64', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb65', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb66', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb67', 'ffff'), 'bit', OPTYPE_const, 4, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb68', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb69', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb6a', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb6b', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb6c', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb6d', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb6e', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb6f', 'ffff'), 'bit', OPTYPE_const, 5, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb70', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb71', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb72', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb73', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb74', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb75', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb76', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb77', 'ffff'), 'bit', OPTYPE_const, 6, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb78', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb79', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb7a', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb7b', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb7c', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb7d', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb7e', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb7f', 'ffff'), 'bit', OPTYPE_const, 7, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb80', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb81', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb82', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb83', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb84', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb85', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb86', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb87', 'ffff'), 'res', OPTYPE_const, 0, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb88', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb89', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb8a', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb8b', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb8c', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb8d', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb8e', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb8f', 'ffff'), 'res', OPTYPE_const, 1, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb90', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb91', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb92', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb93', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb94', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb95', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb96', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb97', 'ffff'), 'res', OPTYPE_const, 2, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cb98', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cb99', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cb9a', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cb9b', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cb9c', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cb9d', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cb9e', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cb9f', 'ffff'), 'res', OPTYPE_const, 3, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cba0', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cba1', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cba2', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cba3', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cba4', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cba5', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cba6', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cba7', 'ffff'), 'res', OPTYPE_const, 4, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cba8', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cba9', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbaa', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbab', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbac', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbad', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbae', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbaf', 'ffff'), 'res', OPTYPE_const, 5, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbb0', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbb1', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbb2', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbb3', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbb4', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbb5', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbb6', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbb7', 'ffff'), 'res', OPTYPE_const, 6, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbb8', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbb9', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbba', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbbb', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbbc', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbbd', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbbe', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbbf', 'ffff'), 'res', OPTYPE_const, 7, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbc0', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbc1', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbc2', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbc3', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbc4', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbc5', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbc6', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbc7', 'ffff'), 'set', OPTYPE_const, 0, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbc8', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbc9', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbca', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbcb', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbcc', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbcd', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbce', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbcf', 'ffff'), 'set', OPTYPE_const, 1, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbd0', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbd1', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbd2', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbd3', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbd4', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbd5', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbd6', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbd7', 'ffff'), 'set', OPTYPE_const, 2, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbd8', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbd9', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbda', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbdb', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbdc', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbdd', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbde', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbdf', 'ffff'), 'set', OPTYPE_const, 3, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbe0', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbe1', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbe2', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbe3', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbe4', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbe5', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbe6', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbe7', 'ffff'), 'set', OPTYPE_const, 4, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbe8', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbe9', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbea', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbeb', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbec', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbed', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbee', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbef', 'ffff'), 'set', OPTYPE_const, 5, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbf0', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbf1', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbf2', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbf3', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbf4', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbf5', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbf6', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbf7', 'ffff'), 'set', OPTYPE_const, 6, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cbf8', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_B, 2, 0, 0),
(('cbf9', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_C, 2, 0, 0),
(('cbfa', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_D, 2, 0, 0),
(('cbfb', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_E, 2, 0, 0),
(('cbfc', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_H, 2, 0, 0),
(('cbfd', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_L, 2, 0, 0),
(('cbfe', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_RegMem, REG_HL, 2, 0, 0),
(('cbff', 'ffff'), 'set', OPTYPE_const, 7, OPTYPE_Reg, REG_A, 2, 0, 0),
(('cc0000', 'ff0000'), 'call', OPTYPE_Cond, COND_Z, OPTYPE_imm16, None, 3, 1, 0),
(('cd0000', 'ff0000'), 'call', OPTYPE_imm16, None, None, None, 3, 1, 0),
(('ce00', 'ff00'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_imm8, None, 2, 1, 0),
(('cf', 'ff'), 'rst', OPTYPE_const, 8, None, None, 1, 0, 0),
(('d0', 'ff'), 'ret', OPTYPE_Cond, COND_NC, None, None, 1, 0, 0),
(('d1', 'ff'), 'pop', OPTYPE_Reg, REG_DE, None, None, 1, 0, 0),
(('d2', 'ff'), 'jp', OPTYPE_Cond, COND_NC, OPTYPE_Ind, None, 1, 0, 0),
(('d300', 'ff00'), 'out', OPTYPE_immmem8, None, OPTYPE_Reg, REG_A, 2, 1, 0),
(('d40000', 'ff0000'), 'call', OPTYPE_Cond, COND_NC, OPTYPE_imm16, None, 3, 1, 0),
(('d40000', 'ff0000'), 'call', OPTYPE_Cond, COND_NC, OPTYPE_imm16, None, 3, 1, 0),
(('d5', 'ff'), 'push', OPTYPE_Reg, REG_DE, None, None, 1, 0, 0),
(('d600', 'ff00'), 'sub', OPTYPE_imm8, None, None, None, 2, 1, 0),
(('d7', 'ff'), 'rst', OPTYPE_const, 16, None, None, 1, 0, 0),
(('d8', 'ff'), 'ret', OPTYPE_Reg, REG_C, None, None, 1, 0, 0),
(('d9', 'ff'), 'exx', None, None, None, None, 1, 0, 0),
(('da', 'ff'), 'jp', OPTYPE_Reg, REG_C, OPTYPE_Ind, None, 1, 0, 0),
(('db00', 'ff00'), 'in', OPTYPE_Reg, REG_A, OPTYPE_immmem8, None, 2, 1, 0),
(('dc0000', 'ff0000'), 'call', OPTYPE_Reg, REG_C, OPTYPE_imm16, None, 3, 1, 0),
(('dd09', 'ffff'), 'add', OPTYPE_Reg, REG_IX, OPTYPE_Reg, REG_BC, 2, 0, 0),
(('dd19', 'ffff'), 'add', OPTYPE_Reg, REG_IX, OPTYPE_Reg, REG_DE, 2, 0, 0),
(('dd210000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_IX, OPTYPE_imm16, None, 4, 2, 0),
(('dd220000', 'ffff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_IX, 4, 2, 0),
(('dd23', 'ffff'), 'inc', OPTYPE_Reg, REG_IX, None, None, 2, 0, 0),
(('dd29', 'ffff'), 'add', OPTYPE_Reg, REG_IX, OPTYPE_Reg, REG_IX, 2, 0, 0),
(('dd2a0000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_IX, OPTYPE_immmem16, None, 4, 2, 0),
(('dd2b', 'ffff'), 'dec', OPTYPE_Reg, REG_IX, None, None, 2, 0, 0),
(('dd3400', 'ffff00'), 'inc', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('dd3500', 'ffff00'), 'dec', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('dd360000', 'ffff0000'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_imm8, None, 4, 2, 0),
(('dd39', 'ffff'), 'add', OPTYPE_Reg, REG_IX, OPTYPE_Reg, REG_SP, 2, 0, 0),
(('dd4600', 'ffff00'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd4e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd5600', 'ffff00'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd5e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd6600', 'ffff00'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd6e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd7000', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_B, 3, 2, 0),
(('dd7100', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_C, 3, 2, 0),
(('dd7200', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_D, 3, 2, 0),
(('dd7300', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_E, 3, 2, 0),
(('dd7400', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_H, 3, 2, 0),
(('dd7500', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_L, 3, 2, 0),
(('dd7700', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IX, OPTYPE_Reg, REG_A, 3, 2, 0),
(('dd7e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd8600', 'ffff00'), 'add', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd8e00', 'ffff00'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dd9600', 'ffff00'), 'sub', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('dd9e00', 'ffff00'), 'sbc', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IX, 3, 2, 0),
(('dda600', 'ffff00'), 'and', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('ddae00', 'ffff00'), 'xor', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('ddb600', 'ffff00'), 'or', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('ddbe00', 'ffff00'), 'cp', OPTYPE_RegMemDisp, REG_IX, None, None, 3, 2, 0),
(('ddcb0006', 'ffff00ff'), 'rlc', OPTYPE_RegMemDisp, REG_IX, None, None, 4, 2, 0),
(('ddcb000e', 'ffff00ff'), 'rrc', OPTYPE_RegMemDisp, REG_IX, None, None, 4, 2, 0),
(('ddcb0016', 'ffff00ff'), 'rl', OPTYPE_RegMemDisp, REG_IX, None, None, 4, 2, 0),
(('ddcb001e', 'ffff00ff'), 'rr', OPTYPE_RegMemDisp, REG_IX, None, None, 4, 2, 0),
(('ddcb0026', 'ffff00ff'), 'sla', OPTYPE_RegMemDisp, REG_IX, None, None, 4, 2, 0),
(('ddcb002e', 'ffff00ff'), 'sra', OPTYPE_RegMemDisp, REG_IX, None, None, 4, 2, 0),
(('ddcb0046', 'ffff00ff'), 'bit', OPTYPE_const, 0, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb004e', 'ffff00ff'), 'bit', OPTYPE_const, 1, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb0056', 'ffff00ff'), 'bit', OPTYPE_const, 2, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb005e', 'ffff00ff'), 'bit', OPTYPE_const, 3, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb0066', 'ffff00ff'), 'bit', OPTYPE_const, 4, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb006e', 'ffff00ff'), 'bit', OPTYPE_const, 5, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb0076', 'ffff00ff'), 'bit', OPTYPE_const, 6, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb007e', 'ffff00ff'), 'bit', OPTYPE_const, 7, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb0086', 'ffff00ff'), 'res', OPTYPE_const, 0, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb008e', 'ffff00ff'), 'res', OPTYPE_const, 1, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb0096', 'ffff00ff'), 'res', OPTYPE_const, 2, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb009e', 'ffff00ff'), 'res', OPTYPE_const, 3, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00a6', 'ffff00ff'), 'res', OPTYPE_const, 4, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00ae', 'ffff00ff'), 'res', OPTYPE_const, 5, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00b6', 'ffff00ff'), 'res', OPTYPE_const, 6, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00be', 'ffff00ff'), 'res', OPTYPE_const, 7, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00c6', 'ffff00ff'), 'set', OPTYPE_const, 0, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00ce', 'ffff00ff'), 'set', OPTYPE_const, 1, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00d6', 'ffff00ff'), 'set', OPTYPE_const, 2, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00de', 'ffff00ff'), 'set', OPTYPE_const, 3, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00e6', 'ffff00ff'), 'set', OPTYPE_const, 4, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00ee', 'ffff00ff'), 'set', OPTYPE_const, 5, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00f6', 'ffff00ff'), 'set', OPTYPE_const, 6, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('ddcb00fe', 'ffff00ff'), 'set', OPTYPE_const, 7, OPTYPE_RegMemDisp, REG_IX, 4, 2, 0),
(('dde1', 'ffff'), 'pop', OPTYPE_Reg, REG_IX, None, None, 2, 0, 0),
(('dde3', 'ffff'), 'ex', OPTYPE_RegMem, REG_SP, OPTYPE_Reg, REG_IX, 2, 0, 0),
(('dde5', 'ffff'), 'push', OPTYPE_Reg, REG_IX, None, None, 2, 0, 0),
(('dde9', 'ffff'), 'jp', OPTYPE_RegMem, REG_IX, None, None, 2, 0, 0),
(('ddf9', 'ffff'), 'ld', OPTYPE_Reg, REG_SP, OPTYPE_Reg, REG_IX, 2, 0, 0),
(('de00', 'ff00'), 'sbc', OPTYPE_Reg, REG_A, OPTYPE_imm8, None, 2, 1, 0),
(('df', 'ff'), 'rst', OPTYPE_const, 24, None, None, 1, 0, 0),
(('e0', 'ff'), 'ret', OPTYPE_Cond, COND_PO, None, None, 1, 0, 0),
(('e1', 'ff'), 'pop', OPTYPE_Reg, REG_HL, None, None, 1, 0, 0),
(('e2', 'ff'), 'jp', OPTYPE_Cond, COND_PO, OPTYPE_Ind, None, 1, 0, 0),
(('e3', 'ff'), 'ex', OPTYPE_RegMem, REG_SP, OPTYPE_Reg, REG_HL, 1, 0, 0),
(('e40000', 'ff0000'), 'call', OPTYPE_Cond, COND_PO, OPTYPE_imm16, None, 3, 1, 0),
(('e5', 'ff'), 'push', OPTYPE_Reg, REG_HL, None, None, 1, 0, 0),
(('e600', 'ff00'), 'and', OPTYPE_imm8, None, None, None, 2, 1, 0),
(('e7', 'ff'), 'rst', OPTYPE_const, 32, None, None, 1, 0, 0),
(('e8', 'ff'), 'ret', OPTYPE_Cond, COND_PE, None, None, 1, 0, 0),
(('e9', 'ff'), 'jp', OPTYPE_RegMem, REG_HL, None, None, 1, 0, 0),
(('ea', 'ff'), 'jp', OPTYPE_Cond, COND_PE, OPTYPE_Ind, None, 1, 0, 0),
(('eb', 'ff'), 'ex', OPTYPE_Reg, REG_DE, OPTYPE_Reg, REG_HL, 1, 0, 0),
(('ec0000', 'ff0000'), 'call', OPTYPE_Cond, COND_PE, OPTYPE_imm16, None, 3, 1, 0),
(('ed40', 'ffff'), 'in', OPTYPE_Reg, REG_B, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed41', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_B, 2, 0, 0),
(('ed42', 'ffff'), 'sbc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_BC, 2, 0, 0),
(('ed430000', 'ffff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_BC, 4, 2, 0),
(('ed44', 'ffff'), 'neg', None, None, None, None, 2, 0, 0),
(('ed45', 'ffff'), 'retn', None, None, None, None, 2, 0, 0),
(('ed46', 'ffff'), 'im', OPTYPE_const, 0, None, None, 2, 0, 0),
(('ed47', 'ffff'), 'ld', OPTYPE_Reg, REG_I, OPTYPE_Reg, REG_A, 2, 0, 0),
(('ed48', 'ffff'), 'in', OPTYPE_Reg, REG_C, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed49', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_C, 2, 0, 0),
(('ed4a', 'ffff'), 'adc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_BC, 2, 0, 0),
(('ed4b0000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_BC, OPTYPE_immmem16, None, 4, 2, 0),
(('ed4d', 'ffff'), 'reti', None, None, None, None, 2, 0, 0),
(('ed50', 'ffff'), 'in', OPTYPE_Reg, REG_D, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed51', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_D, 2, 0, 0),
(('ed52', 'ffff'), 'sbc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_DE, 2, 0, 0),
(('ed530000', 'ffff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_DE, 4, 2, 0),
(('ed56', 'ffff'), 'im', OPTYPE_const, 1, None, None, 2, 0, 0),
(('ed57', 'ffff'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_Reg, REG_I, 2, 0, 0),
(('ed58', 'ffff'), 'in', OPTYPE_Reg, REG_E, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed59', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_E, 2, 0, 0),
(('ed5a', 'ffff'), 'adc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_DE, 2, 0, 0),
(('ed5b0000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_DE, OPTYPE_immmem16, None, 4, 2, 0),
(('ed5e', 'ffff'), 'im', OPTYPE_const, 2, None, None, 2, 0, 0),
(('ed60', 'ffff'), 'in', OPTYPE_Reg, REG_H, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed61', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_H, 2, 0, 0),
(('ed62', 'ffff'), 'sbc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_HL, 2, 0, 0),
(('ed67', 'ffff'), 'rrd', None, None, None, None, 2, 0, 0),
(('ed68', 'ffff'), 'in', OPTYPE_Reg, REG_L, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed69', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_L, 2, 0, 0),
(('ed6a', 'ffff'), 'adc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_HL, 2, 0, 0),
(('ed6f', 'ffff'), 'rld', None, None, None, None, 2, 0, 0),
(('ed72', 'ffff'), 'sbc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_SP, 2, 0, 0),
(('ed730000', 'ffff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_SP, 4, 2, 0),
(('ed78', 'ffff'), 'in', OPTYPE_Reg, REG_A, OPTYPE_RegMem, REG_C, 2, 0, 0),
(('ed79', 'ffff'), 'out', OPTYPE_RegMem, REG_C, OPTYPE_Reg, REG_A, 2, 0, 0),
(('ed7a', 'ffff'), 'adc', OPTYPE_Reg, REG_HL, OPTYPE_Reg, REG_SP, 2, 0, 0),
(('ed7b0000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_SP, OPTYPE_immmem16, None, 4, 2, 0),
(('eda0', 'ffff'), 'ldi', None, None, None, None, 2, 0, 0),
(('eda1', 'ffff'), 'cpi', None, None, None, None, 2, 0, 0),
(('eda2', 'ffff'), 'ini', None, None, None, None, 2, 0, 0),
(('eda3', 'ffff'), 'outi', None, None, None, None, 2, 0, 0),
(('eda8', 'ffff'), 'ldd', None, None, None, None, 2, 0, 0),
(('eda9', 'ffff'), 'cpd', None, None, None, None, 2, 0, 0),
(('edaa', 'ffff'), 'ind', None, None, None, None, 2, 0, 0),
(('edab', 'ffff'), 'outd', None, None, None, None, 2, 0, 0),
(('edb0', 'ffff'), 'ldir', None, None, None, None, 2, 0, 0),
(('edb1', 'ffff'), 'cpir', None, None, None, None, 2, 0, 0),
(('edb2', 'ffff'), 'inir', None, None, None, None, 2, 0, 0),
(('edb3', 'ffff'), 'otir', None, None, None, None, 2, 0, 0),
(('edb8', 'ffff'), 'lddr', None, None, None, None, 2, 0, 0),
(('edb9', 'ffff'), 'cpdr', None, None, None, None, 2, 0, 0),
(('edba', 'ffff'), 'indr', None, None, None, None, 2, 0, 0),
(('edbb', 'ffff'), 'otdr', None, None, None, None, 2, 0, 0),
(('ee00', 'ff00'), 'xor', OPTYPE_imm8, None, None, None, 2, 1, 0),
(('ef', 'ff'), 'rst', OPTYPE_const, 40, None, None, 1, 0, 0),
(('f0', 'ff'), 'ret', OPTYPE_Cond, COND_P, None, None, 1, 0, 0),
(('f1', 'ff'), 'pop', OPTYPE_Reg, REG_AF, None, None, 1, 0, 0),
(('f2', 'ff'), 'jp', OPTYPE_Cond, COND_P, OPTYPE_Ind, None, 1, 0, 0),
(('f3', 'ff'), 'di', None, None, None, None, 1, 0, 0),
(('f40000', 'ff0000'), 'call', OPTYPE_Cond, COND_P, OPTYPE_imm16, None, 3, 1, 0),
(('f5', 'ff'), 'push', OPTYPE_Reg, REG_AF, None, None, 1, 0, 0),
(('f600', 'ff00'), 'or', OPTYPE_imm8, None, None, None, 2, 1, 0),
(('f7', 'ff'), 'rst', OPTYPE_const, 48, None, None, 1, 0, 0),
(('f8', 'ff'), 'ret', OPTYPE_Cond, COND_M, None, None, 1, 0, 0),
(('f9', 'ff'), 'ld', OPTYPE_Reg, REG_SP, OPTYPE_Reg, REG_HL, 1, 0, 0),
(('fa', 'ff'), 'jp', OPTYPE_Cond, COND_M, OPTYPE_Ind, None, 1, 0, 0),
(('fb', 'ff'), 'ei', None, None, None, None, 1, 0, 0),
(('fc0000', 'ff0000'), 'call', OPTYPE_Cond, COND_M, OPTYPE_imm16, None, 3, 1, 0),
(('fd09', 'ffff'), 'add', OPTYPE_Reg, REG_IY, OPTYPE_Reg, REG_BC, 2, 0, 0),
(('fd19', 'ffff'), 'add', OPTYPE_Reg, REG_IY, OPTYPE_Reg, REG_DE, 2, 0, 0),
(('fd210000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_IY, OPTYPE_imm16, None, 4, 2, 0),
(('fd220000', 'ffff0000'), 'ld', OPTYPE_immmem16, None, OPTYPE_Reg, REG_IY, 4, 2, 0),
(('fd23', 'ffff'), 'inc', OPTYPE_Reg, REG_IY, None, None, 2, 0, 0),
(('fd29', 'ffff'), 'add', OPTYPE_Reg, REG_IY, OPTYPE_Reg, REG_IY, 2, 0, 0),
(('fd2a0000', 'ffff0000'), 'ld', OPTYPE_Reg, REG_IY, OPTYPE_immmem16, None, 4, 2, 0),
(('fd2b', 'ffff'), 'dec', OPTYPE_Reg, REG_IY, None, None, 2, 0, 0),
(('fd3400', 'ffff00'), 'inc', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fd3500', 'ffff00'), 'dec', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fd360000', 'ffff0000'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_imm8, None, 4, 2, 0),
(('fd39', 'ffff'), 'add', OPTYPE_Reg, REG_IY, OPTYPE_Reg, REG_SP, 2, 0, 0),
(('fd4600', 'ffff00'), 'ld', OPTYPE_Reg, REG_B, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd4e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_C, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd5600', 'ffff00'), 'ld', OPTYPE_Reg, REG_D, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd5e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_E, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd6600', 'ffff00'), 'ld', OPTYPE_Reg, REG_H, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd6e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_L, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd7000', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_B, 3, 2, 0),
(('fd7100', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_C, 3, 2, 0),
(('fd7200', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_D, 3, 2, 0),
(('fd7300', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_E, 3, 2, 0),
(('fd7400', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_H, 3, 2, 0),
(('fd7500', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_L, 3, 2, 0),
(('fd7700', 'ffff00'), 'ld', OPTYPE_RegMemDisp, REG_IY, OPTYPE_Reg, REG_A, 3, 2, 0),
(('fd7e00', 'ffff00'), 'ld', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd8600', 'ffff00'), 'add', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd8e00', 'ffff00'), 'adc', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fd9600', 'ffff00'), 'sub', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fd9e00', 'ffff00'), 'sbc', OPTYPE_Reg, REG_A, OPTYPE_RegMemDisp, REG_IY, 3, 2, 0),
(('fda600', 'ffff00'), 'and', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fdae00', 'ffff00'), 'xor', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fdb600', 'ffff00'), 'or', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fdbe00', 'ffff00'), 'cp', OPTYPE_RegMemDisp, REG_IY, None, None, 3, 2, 0),
(('fdcb0006', 'ffff00ff'), 'rlc', OPTYPE_RegMemDisp, REG_IY, None, None, 4, 2, 0),
(('fdcb000e', 'ffff00ff'), 'rrc', OPTYPE_RegMemDisp, REG_IY, None, None, 4, 2, 0),
(('fdcb0016', 'ffff00ff'), 'rl', OPTYPE_RegMemDisp, REG_IY, None, None, 4, 2, 0),
(('fdcb001e', 'ffff00ff'), 'rr', OPTYPE_RegMemDisp, REG_IY, None, None, 4, 2, 0),
(('fdcb0026', 'ffff00ff'), 'sla', OPTYPE_RegMemDisp, REG_IY, None, None, 4, 2, 0),
(('fdcb002e', 'ffff00ff'), 'sra', OPTYPE_RegMemDisp, REG_IY, None, None, 4, 2, 0),
(('fdcb0046', 'ffff00ff'), 'bit', OPTYPE_const, 0, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb004e', 'ffff00ff'), 'bit', OPTYPE_const, 1, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb0056', 'ffff00ff'), 'bit', OPTYPE_const, 2, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb005e', 'ffff00ff'), 'bit', OPTYPE_const, 3, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb0066', 'ffff00ff'), 'bit', OPTYPE_const, 4, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb006e', 'ffff00ff'), 'bit', OPTYPE_const, 5, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb0076', 'ffff00ff'), 'bit', OPTYPE_const, 6, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb007e', 'ffff00ff'), 'bit', OPTYPE_const, 7, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb0086', 'ffff00ff'), 'res', OPTYPE_const, 0, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb008e', 'ffff00ff'), 'res', OPTYPE_const, 1, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb0096', 'ffff00ff'), 'res', OPTYPE_const, 2, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb009e', 'ffff00ff'), 'res', OPTYPE_const, 3, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00a6', 'ffff00ff'), 'res', OPTYPE_const, 4, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00ae', 'ffff00ff'), 'res', OPTYPE_const, 5, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00b6', 'ffff00ff'), 'res', OPTYPE_const, 6, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00be', 'ffff00ff'), 'res', OPTYPE_const, 7, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00c6', 'ffff00ff'), 'set', OPTYPE_const, 0, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00ce', 'ffff00ff'), 'set', OPTYPE_const, 1, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00d6', 'ffff00ff'), 'set', OPTYPE_const, 2, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00de', 'ffff00ff'), 'set', OPTYPE_const, 3, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00e6', 'ffff00ff'), 'set', OPTYPE_const, 4, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00ee', 'ffff00ff'), 'set', OPTYPE_const, 5, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00f6', 'ffff00ff'), 'set', OPTYPE_const, 6, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fdcb00fe', 'ffff00ff'), 'set', OPTYPE_const, 7, OPTYPE_RegMemDisp, REG_IY, 4, 2, 0),
(('fde1', 'ffff'), 'pop', OPTYPE_Reg, REG_IY, None, None, 2, 0, 0),
(('fde3', 'ffff'), 'ex', OPTYPE_RegMem, REG_SP, OPTYPE_Reg, REG_IY, 2, 0, 0),
(('fde5', 'ffff'), 'push', OPTYPE_Reg, REG_IY, None, None, 2, 0, 0),
(('fde9', 'ffff'), 'jp', OPTYPE_RegMem, REG_IY, None, None, 2, 0, 0),
(('fdf9', 'ffff'), 'ld', OPTYPE_Reg, REG_SP, OPTYPE_Reg, REG_IY, 2, 0, 0),
(('fe00', 'ff00'), 'cp', OPTYPE_imm8, None, None, None, 2, 1, 0),
(('ff', 'ff'), 'rst', OPTYPE_const, 56, None, None, 1, 0, 0),
]

from envi.bits import binbytes as bb

nbits = '00000000'
nnbits = '0000000000000000'

newtab = (

  # 8bit ld variants...
  (bb('01000000'), bb('11000000'), 'ld', OPTYPE_Reg, (2, 3), OPTYPE_Reg, (5, 3)),
  (bb('00000110'), bb('11000111'), 'ld', OPTYPE_Reg, (2, 3), OPTYPE_Imm, (8, 8)),
  (bb('01000110'), bb('11000111'), 'ld', OPTYPE_Reg, (2, 3), OPTYPE_OpReg, REG_HL),
  (bb('110111010100011000000000'), bb('111111111100011100000000'), 'ld', OPTYPE_Reg, (10, 3), OPTYPE_RegMemDisp, (REG_IX, 16, 8)),
  (bb('111111010100011000000000'), bb('111111111100011100000000'), 'ld', OPTYPE_Reg, (10, 3), OPTYPE_RegMemDisp, (REG_IY, 16, 8)),
  (bb('01110000'), bb('11111000'), 'ld', OPTYPE_RegMem, (REG_HL, None, None), OPTYPE_Reg, (5, 3)),
  (bb('110111010111000000000000'), bb('111111111111100000000000'), 'ld', OPTYPE_RegMemDisp, (REG_IX, 16, 8), OPTYPE_Reg, (13, 3)),
  (bb('111111010111000000000000'), bb('111111111111100000000000'), 'ld', OPTYPE_RegMemDisp, (REG_IY, 16, 8), OPTYPE_Reg, (13, 3)),
  (bb('0011011000000000'), bb('1111111100000000'), 'ld', OPTYPE_RegMem, (REG_HL, None, None), OPTYPE_Imm, (8, 8)),
  (bb('11011101001101100000000000000000'), bb('11111111111111110000000000000000'), 'ld', OPTYPE_RegMemDisp, (REG_IX, 16, 8), OPTYPE_Imm, (24, 8)),
  (bb('11111101001101100000000000000000'), bb('11111111111111110000000000000000'), 'ld', OPTYPE_RegMemDisp, (REG_IY, 16, 8), OPTYPE_Imm, (24, 8)),
  (bb('00001010'), bb('11111111'), 'ld', OPTYPE_OpReg, REG_A, OPTYPE_RegMem, (REG_BC, None, None)),
  (bb('00011010'), bb('11111111'), 'ld', OPTYPE_OpReg, REG_A, OPTYPE_RegMem, (REG_DE, None, None)),
  (bb('001110100000000000000000'), bb('111111110000000000000000'), 'ld', OPTYPE_OpReg, REG_A, OPTYPE_ImmMem, (8, 16)),
  (bb('00000010'), bb('11111111'), 'ld', OPTYPE_RegMem, (REG_BC, None, None), OPTYPE_OpReg, REG_A),
  (bb('00010010'), bb('11111111'), 'ld', OPTYPE_RegMem, (REG_DE, None, None), OPTYPE_OpReg, REG_A),
  (bb('001100100000000000000000'), bb('111111110000000000000000'), 'ld', OPTYPE_ImmMem, (8, 16), OPTYPE_OpReg, REG_A),
  (bb('1110110101010111'), bb('1111111111111111'), 'ld', OPTYPE_OpReg, REG_A, OPTYPE_OpReg, REG_I),
  (bb('1110110101011111'), bb('1111111111111111'), 'ld', OPTYPE_OpReg, REG_A, OPTYPE_OpReg, REG_R),
  (bb('1110110101000111'), bb('1111111111111111'), 'ld', OPTYPE_OpReg, REG_I, OPTYPE_OpReg, REG_A),
  (bb('111011010100f111'), bb('1111111111111111'), 'ld', OPTYPE_OpReg, REG_R, OPTYPE_OpReg, REG_A),

  # 16 bit ld variants
  (bb('000000010000000000000000'), bb('110011110000000000000000'), 'ld', OPTYPE_RegPair, (2, 2), OPTYPE_Imm, (8, 16)),
  (bb('11011101001000010000000000000000'), bb('11111111111111110000000000000000'), 'ld', OPTYPE_OpReg, REG_IX, OPTYPE_Imm, (16, 16)),
  (bb('11111101001000010000000000000000'), bb('11111111111111110000000000000000'), 'ld', OPTYPE_OpReg, REG_IY, OPTYPE_Imm, (16, 16)),
  (bb('001010100000000000000000'), bb('111111110000000000000000'), 'ld', OPTYPE_OpReg, REG_HL, OPTYPE_ImmMem, (8,16)),
  (bb('11101101010010110000000000000000'), bb('11111111110011110000000000000000'), 'ld', OPTYPE_RegPair, (10, 2), OPTYPE_ImmMem, (16, 16)),
  (bb('11011101001010100000000000000000'), bb('11111111111111110000000000000000'), 'ld', OPTYPE_OpReg, REG_IX, OPTYPE_ImmMem, (16, 16)),
  (bb('11111101001010100000000000000000'), bb('11111111111111110000000000000000'), 'ld', OPTYPE_OpReg, REG_IY, OPTYPE_ImmMem, (16, 16)),
  (bb('001000100000000000000000'), bb('111111110000000000000000'), 'ld', OPTYPE_ImmMem, (8, 16), OPTYPE_OpReg, REG_HL),
)




import envi.archs.i386 as e_i386
import vivisect.impemu.emulator as v_i_emulator

from envi.const import RMETA_NMASK

non_use_mnems = ('push', )


class i386WorkspaceEmulator(v_i_emulator.WorkspaceEmulator, e_i386.IntelEmulator):
    __archemu__ = e_i386.IntelEmulator

    taintregs = [
        e_i386.REG_EAX, e_i386.REG_ECX, e_i386.REG_EDX,
        e_i386.REG_EBX, e_i386.REG_EBP, e_i386.REG_ESI,
        e_i386.REG_EDI,
    ]

    def __init__(self, vw, **kwargs):
        '''
        Please see the base emulator class in vivisect/impemu/emulator.py for the parameters
        that can be passed through kwargs
        '''
        e_i386.IntelEmulator.__init__(self)
        v_i_emulator.WorkspaceEmulator.__init__(self, vw, **kwargs)
        self.setEmuOpt('i386:repmax', 1)

    def getRegister(self, index):
        rval = value = e_i386.IntelEmulator.getRegister(self, index)

        if self.op is None:
            return value

        ridx = index
        if self.isMetaRegister(index):
            ridx = index & RMETA_NMASK
            # use the value of the real register (not the meta register) for _useVirtAddr
            # but ultimately return the meta register value
            rval = e_i386.IntelEmulator.getRegister(self, ridx)

        if ridx not in self.taintregs:
            return value

        if self.isRegUse(self.op, ridx):
            self._useVirtAddr(rval)

        return value

    def isRegUse(self, op, ridx):
        '''
        determines if the sequence of uses(get/setRegister) is a register 'use'.
        '''
        # conditions that indicate 'usage':
        # * 'normal usage'
        #   read from a register with *no* previous writes/sets to that register

        # conditions that do *not* indicate 'usage':
        # * 'register initialization', ie xor rax, rax, mov rax, 0, etc
        # * 'save/restore of register', ie push/pop

        # blacklist of mnems that do *not* indicate a 'use'

        # if first item is a set, then this is *not* an arg(register) use
        # (reg initialization)
        if op.mnem == 'xor' and op.opers[0] == op.opers[1]:
            # xor register initialization
            return False

        else:
            # if op mnem is in blacklist, it's not a use either
            if op.mnem in non_use_mnems:
                return False

        return True



import vstruct
import vivisect
import envi.memcanvas as e_mcanvas
import vstruct.primitives as v_prim
import vivisect.renderers as v_rend
import vivisect.tests.utils as v_t_utils

class VivisectVstructTest(v_t_utils.VivTest):

    def test_render_vstruct(self):
        structcmp = '''.dynsym:0x000099e0  Elf32Symbol: 
.dynsym:0x000099e0    st_name:     0x5ba   (1466)
.dynsym:0x000099e4    st_value:    0x21769   (137065)
.dynsym:0x000099e8    st_size:     0x6c   (108)
.dynsym:0x000099ec    st_info:     0x12   (18)
.dynsym:0x000099ed    st_other:    0x0   (0)
.dynsym:0x000099ee    st_shndx:    0xb   (11)
'''

        vw = vivisect.VivWorkspace()
        vw.setMeta('Architecture', 'i386')
        canvas = e_mcanvas.StringMemoryCanvas(vw)
        vw.addSegment(0x1000, 0x9000, '.dynsym', 'foobar')
        vw.addMemoryMap(0x99e0, 7, 'foobar', b'\xba\x05\x00\x00\x69\x17\x02\x00\x6c\x00\x00\x00\x12\x00\x0b' + b'\0'*1000)
        wsr = v_rend.WorkspaceRenderer(vw)

        mod = vw.loadModule('vstruct.defs.elf')
        vw.vsbuilder.addVStructNamespace('elf', mod)
        vw.makeStructure(0x99e0, 'elf.Elf32Symbol')
        wsr = v_rend.WorkspaceRenderer(vw)
        loc = vw.getLocation(0x99e0)

        struct = vw.getStructure(0x99e0, loc[vivisect.L_TINFO])
        wsr.renderLocation(canvas, loc, None, False, 'cmnt', struct)
        self.maxDiff=1000
        self.assertEqual(canvas.strval, structcmp)



from envi.const import *
from vivisect.symboliks.common import *

class ArgDefSymEmu(object):
    '''
    An emulator snapped in to return the symbolic representation of
    a calling convention arg definition.  This is used by getPreCallArgs when
    called by {get, set}SymbolikArgs.  This allows us to not have to
    re-implement cc argument parsing *again* for symbolics.
    '''
    def __init__(self, xlator=None):
        if not xlator:
            xlator = self.__xlator__(None)
        self.xlator = xlator

    def getRegister(self, ridx):
        '''
        uses the translators method to return a symbolic object for a
        register index.
        '''
        return self.xlator.getRegObj(ridx)

    def getStackCounter(self):
        '''
        uses the translators register ctx to find the sp index and returns a
        symbolic object for the sp.
        '''
        return self.getRegister(self.xlator._reg_ctx._rctx_spindex)

    def setStackCounter(self, value):
        '''
        stubbed out so when we re-use deallocateCallSpace we don't get an
        exception.  we only use the delta returned by deallocateCallSpace; we
        don't care that it's setting the stack counter.
        '''
        pass

    def readMemoryFormat(self, va, fmt):
        # TODO: we assume psize and le, better way? must be...
        if not fmt.startswith('<') and not fmt.endswith('P'):
            raise Exception('we dont handle this format string')

        if isinstance(va, int):
            va = Const(va, self.xlator._psize)

        if len(fmt) == 2:
            return Mem(va, Const(self.xlator._psize, self.xlator._psize))

        args = []
        num = int(fmt[1:fmt.index('P')])
        for i in range(num):
            args.append(Mem(va, Const(self.xlator._psize, self.xlator._psize)))
            va += Const(self.xlator._psize, self.xlator._psize)

        return args

class SymbolikCallingConvention(object):
    '''
    use as a mixin to an existing envi calling convention.
    '''
    def __init__(self, xlator=None):
        self.argdefemu = self.__argdefemu__(xlator)
        # Set up the symbolic constants
        self._width = self.align

        self.pad = Const(self.pad, self._width)
        self.align = Const(self.align, self._width)
        self.delta = Const(self.delta, self._width)

    def _dealloc(self, delta, argc):
        # Special method to allow symbolik cconv to hook...
        return delta + (self.align * Const(argc, self._width))

    def getSymbolikArgs(self, emu, argv, update=False):
        '''
        Used when symbolik emulation discovers a call while doing emulation.
        This allows per-arch-calling-convention argument parsing for updates to
        fofx effects...
        '''
        args = self.getPreCallArgs(self.argdefemu, len(argv))
        if update:
            args = [arg.update(emu) for arg in args]

        return args

    def setSymbolikArgs(self, emu, argv):
        '''
        Setup the emulator for emulation of a call to a function with this
        calling convention and the specified arguments in argv.

        Example:
            cconv.setSymbolikArgs(emu, [sym1, sym2])
        '''
        arglocs = self.getPreCallArgs(self.argdefemu, len(argv))
        for idx, arg in enumerate(argv):
            argloc = arglocs[idx]

            if isinstance(argloc, Var):
                emu.setSymVariable(argloc.name, arg)

            elif isinstance(argloc, Mem):
                # writeSymMemory takes an symaddr NOT a Mem object.  therefore
                # we get the symaddr out of the Mem object and update it's
                # state using the emulator.  then we add to it any alignment
                # specified by the calling convention.  normally we'd use the
                # getCallArgs API instead of getPreCallArgs, but getCallArgs
                # adjusts the sp using getStackCounter which we cannot easily
                # hook in this context. (and don't really want to add
                # additional effects anyway)
                argloc = argloc.kids[0].update(emu=emu)
                emu.writeSymMemory(argloc + self.align, arg)
            else:
                raise Exception('invalid arg location')

    def setSymbolikReturn(self, emu, sym, argv, precall=False):
        '''
        Set the fofx() return state in the calling emulator to reflect that
        state introduced by our callee.

        Example:
            cconv.setSymbolikReturn(emu, Var('foo', self._psize))
        '''
        # TODO: consider renaming func name (see new envi cc stuff)
        # we could get more re-use here if we plumb/snap in methods that map
        # setReturnValue -> setSymVariable, etc.
        atype, aval = self.retval_def
        if atype != CC_REG:
            raise Exception('implement non-register symbolik return vals')

        rname = self.argdefemu.xlator._reg_ctx.getRegisterName(aval)
        emu.setSymVariable(rname, sym)

        spdelta = self.deallocateCallSpace(self.argdefemu, len(argv), precall=precall)

        spidx = self.argdefemu.xlator._reg_ctx._rctx_spindex
        spname = self.argdefemu.xlator._reg_ctx.getRegisterName(spidx)
        sp = emu.getSymVariable(spname)
        emu.setSymVariable(spname, sp + spdelta)

    def getSymbolikReturn(self, emu):
        atype, aval = self.retval_def
        if atype != CC_REG:
            raise Exception('implement non-register symbolik return vals')

        rname = self.argdefemu.xlator._reg_ctx.getRegisterName(aval)
        return emu.getSymVariable(rname)



import logging
from binascii import unhexlify

logger = logging.getLogger(__name__)

thunk_lookup = {
    unhexlify('8b0424c3'): 'eax',
    unhexlify('8b0c24c3'): 'ecx',
    unhexlify('8b1424c3'): 'edx',
    unhexlify('8b1c24c3'): 'ebx',
    unhexlify('8b3424c3'): 'esi',
    unhexlify('8b3c24c3'): 'edi',
    unhexlify('8b2c24c3'): 'ebp',
}

def analyzeFunction(vw, fva):
    '''
    this analysis module will identify thunk_reg functions, which take the return value and place
    it into EBX.  this is done for position-independent code in i386 elf binaries.  a call to this
    function will be followed by an immediate add to find the start of the module.  that value is
    then used with fixed offsets to access resources within the binary.  it's a bit like the old
    shellcode trick.

    store funcva in "thunk_reg" VaSet in case we identify multiples (not likely) or misidentify
    something.

    then store the module base in metadata as "PIE_ebx", accessible by other analysis modules.
    '''
    sigbytes = vw.readMemory(fva, 4)
    sigreg = thunk_lookup.get(sigbytes)
    logger.debug("0x%x: sigreg: %r", fva, sigreg)

    if sigreg is not None:
        # have we already recorded this thunk_reg?
        if vw.getVaSetRow('thunk_reg', fva) is not None:
            logger.info("Not overriding thunk_reg 0x%x", fva)
            return

        # determine where reg ends up pointing to
        # this requires checking the calling function's next instruction
        refs = vw.getXrefsTo(fva)
        if refs is None or not len(refs):
            logger.info("No xrefs to thunk_reg 0x%x", fva)
            return

        va = refs[0][0]
        op = vw.parseOpcode(va)
        op2 = vw.parseOpcode(va + len(op))
        if op2.mnem != "add":
            logger.warning("Call to thunk_reg not followed by an add: %s", op2)
            return

        addt = op2.opers[1].getOperValue(op2)
        tgtval = op2.va + addt

        logger.debug("__x86.get_pc_thunk.%s: %s", sigreg[1:], hex(tgtval))
        curname = vw.getName(fva)
        if curname is None or curname == "sub_%.8x" % fva:
            vw.makeName(fva, "thunk_%s_%.8x" % (sigreg, fva))

        vw.setVaSetRow('thunk_reg', (fva, sigreg, tgtval))



'''
This directory should contain tidbits and trinkets which help the maintainers upkeep the code.

This repository of information/tools began with commented-out-notes from the symbolic switchcase module, to keep track of things like symbolic states and various kinds of switchcases we've encountered and had to analyze.

vivisect.tests.testelf relies upon a number of python modules which store unittest data.  Those modules are generated by scripts which are ugly and not maintained as part of the official Vivisect code... so we store them here so they're never lost (eg.  atlas gets hit by a bus)
'''



import os
import binascii

import vtrace.tests as vt_tests


class VtraceWritememTest(vt_tests.VtraceProcessTest):

    pypath = os.path.join('vtrace', 'tests', 'mains', 'mainwritemem.py')

    def test_vtrace_writemem(self):

        self.runProcess()

        addrstr = self.proc.stdout.readline()
        addr = int(addrstr, 16)

        testbuf = binascii.hexlify(os.urandom(10))

        # Stop him so we can write to the buffer he created
        self.trace.sendBreak()
        self.trace.writeMemory(addr, testbuf)

        self.runUntilExit()

        # He should now print what we wrote...
        gotline = self.proc.stdout.readline().strip()
        self.assertEqual(testbuf.decode('utf-8'), gotline)



import logging

import vstruct
import vstruct.defs.inet as vs_inet

from vstruct.primitives import *

logger = logging.getLogger(__name__)

PCAP_LINKTYPE_ETHER     = 1
PCAP_LINKTYPE_RAW       = 101
PCAP_LINKTYPE_LINUX_SLL = 113
PCAP_DLT_RAW            = 12

PCAPNG_BOM              = 0x1A2B3C4D
OPT_ENDOFOPT            = 0
OPT_COMMENT             = 1

#PCAPNG_BLOCKTYPE_SECTION_HEADER options
OPT_SHB_HARDWARE        = 2
OPT_SHB_OS              = 3
OPT_SHB_USERAPPL        = 4

#PCAPNG_INTERFACE_DESCRIPTION_BLOCK options
OPT_IF_NAME             = 2
OPT_IF_DESCRIPTION      = 3
OPT_IF_IPV4ADDR         = 4
OPT_IF_IPV6ADDR         = 5
OPT_IF_MACADDR          = 6
OPT_IF_EUIADDR          = 7
OPT_IF_SPEED            = 8
OPT_IF_TSRESOL          = 9
OPT_IF_TZONE            = 10
OPT_IF_FILTER           = 11
OPT_IF_OS               = 12
OPT_IF_FCSLEN           = 13
OPT_IF_TSOFFSET         = 14

# options for PCAPNG_ENHANCED_PACKET_BLOCK
OPT_EPB_FLAGS           = 2
OPT_EPB_HASH            = 3
OPT_EPB_DROPCOUNT       = 4

# values used in the blocktype field
PCAPNG_BLOCKTYPE_INTERFACE_DESCRIPTION      = 0x00000001
PCAPNG_BLOCKTYPE_PACKET                     = 0x00000002
PCAPNG_BLOCKTYPE_SIMPLE_PACKET              = 0x00000003
PCAPNG_BLOCKTYPE_NAME_RESOLUTION            = 0x00000004
PCAPNG_BLOCKTYPE_INTERFACE_STATS            = 0x00000005
PCAPNG_BLOCKTYPE_ENHANCED_PACKET            = 0x00000006
PCAPNG_BLOCKTYPE_SECTION_HEADER             = 0x0a0d0d0a

def pad4bytes(size):
    if (size % 4) == 0:
        return size
    return size + (4 -( size % 4))

class PCAP_FILE_HEADER(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.magic      = v_uint32()
        self.vers_maj   = v_uint16()
        self.vers_min   = v_uint16()
        self.thiszone   = v_uint32()
        self.sigfigs    = v_uint32()
        self.snaplen    = v_uint32()
        self.linktype   = v_uint32()

class PCAP_PACKET_HEADER(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.tvsec      = v_uint32()
        self.tvusec     = v_uint32()
        self.caplen     = v_uint32()
        self.len        = v_uint32()

class PCAPNG_GENERIC_BLOCK_HEADER(vstruct.VStruct):
    '''
    Used to read the block type & size when parsing the file
    '''
    def __init__(self, bigend=False):
        vstruct.VStruct.__init__(self)
        self.blocktype      = v_uint32(bigend=bigend)
        self.blocksize      = v_uint32(bigend=bigend)

class PCAPNG_BLOCK_PARENT(vstruct.VStruct):
    '''
    Used to inherit the weird parsing style where there's variable length
    options at the end, followed by the duplicate block total length
    '''
    def __init__(self, bigend=False):
        vstruct.VStruct.__init__(self)
        #non-vstruct field, set during checking BOM
        self.bigend         = False

    def vsParse(self, bytez, offset=0):
        startoff = offset
        roff = vstruct.VStruct.vsParse(self, bytez, offset=offset)
        #(blocksize-4): because we still need the trailing blocksize2
        # apparently blocks can completely omit the options list and not
        # even have the OPT_ENDOFOPT entry
        while (roff < len(bytez)) and ((roff-startoff) < (self.blocksize-4)):
            opt = PCAPNG_OPTION(bigend=self.bigend)
            roff = opt.vsParse(bytez, roff)
            if opt.code == OPT_ENDOFOPT:
                break
            self.options.vsAddElement(opt)
        # append trailing blocksize2
        bs2 = v_uint32(bigend=self.bigend)
        self.vsAddField('blocksize2', bs2)
        roff = bs2.vsParse(bytez, roff)
        #pad, plus we skip
        return pad4bytes(roff)


class PCAPNG_SECTION_HEADER_BLOCK(PCAPNG_BLOCK_PARENT):
    def __init__(self, bigend=False):
        PCAPNG_BLOCK_PARENT.__init__(self, bigend)
        self.blocktype      = v_uint32(bigend=bigend)
        self.blocksize      = v_uint32(bigend=bigend)
        self.bom            = v_uint32(bigend=bigend)
        self.vers_maj       = v_uint16(bigend=bigend)
        self.vers_min       = v_uint16(bigend=bigend)
        self.sectionsize    = v_uint64(bigend=bigend)
        self.options        = vstruct.VArray([])
        #blocksize2: dynamcally added in vsParse()
        #self.blocksize2     = v_uint32(bigend=bigend)

    def pcb_bom(self):
        bom = self.vsGetField('bom')
        if  self.bom == PCAPNG_BOM:
            #if it matches, then the endian of bom is correct
            self.bigend = bom._vs_bigend
        else:
            self.bigend = not bom._vs_bigend

class PCAPNG_OPTION(vstruct.VStruct):
    def __init__(self, bigend=False):
        vstruct.VStruct.__init__(self)
        self.code           = v_uint16(bigend=bigend)
        self.optsize        = v_uint16(bigend=bigend)
        self.bytes          = v_bytes(0)

    def pcb_optsize(self):
        size = pad4bytes(self.optsize)
        self.vsGetField('bytes').vsSetLength(size)

class PCAPNG_INTERFACE_DESCRIPTION_BLOCK(PCAPNG_BLOCK_PARENT):
    def __init__(self, bigend=False):
        PCAPNG_BLOCK_PARENT.__init__(self, bigend)
        self.blocktype      = v_uint32(bigend=bigend)
        self.blocksize      = v_uint32(bigend=bigend)
        self.linktype       = v_uint16(bigend=bigend)
        self.reserved       = v_uint16(bigend=bigend)
        self.snaplen        = v_uint32(bigend=bigend)
        self.options        = vstruct.VArray([])
        #blocksize2: dynamcally added in vsParse()
        #self.blocksize2     = v_uint32(bigend=bigend)

    def vsParse(self, bytez, offset=0):
        '''
        We need the tsresol value to adjust timestamp values, so pull it
        out here
        '''
        ret = PCAPNG_BLOCK_PARENT.vsParse(self, bytez, offset=0)
        self.tsresol = None
        #default offset is 0
        self.tsoffset = 0
        #sys.stderr.write('PCAPNG_INTERFACE_DESCRIPTION_BLOCK: searching options')
        for i, opt in self.options:
            if opt.code == OPT_IF_TSRESOL:
                self.tsresol = ord(opt.bytes[0])
                #sys.stderr.write('Got tsresol: 0x%x\n' % self.tsresol)
            elif opt.code == OPT_IF_TSOFFSET:
                fmt = '<Q'
                if self.bigend:
                    fmt = '>Q'
                self.tsoffset = struct.unpack_from(fmt, opt.bytes)[0]
                #sys.stderr.write('Got tsoffset: 0x%x\n' % self.tsoffset)
        return ret

class PCAPNG_ENHANCED_PACKET_BLOCK(PCAPNG_BLOCK_PARENT):
    def __init__(self, bigend=False):
        PCAPNG_BLOCK_PARENT.__init__(self, bigend)
        self.blocktype      = v_uint32(bigend=bigend)
        self.blocksize      = v_uint32(bigend=bigend)
        self.interfaceid    = v_uint32(bigend=bigend)
        self.tstamphi       = v_uint32(bigend=bigend)
        self.tstamplow      = v_uint32(bigend=bigend)
        self.caplen         = v_uint32(bigend=bigend)
        self.packetlen      = v_uint32(bigend=bigend)
        self.data           = v_bytes(0)
        self.options        = vstruct.VArray([])
        #blocksize2: dynamcally added in vsParse()
        #self.blocksize2     = v_uint32(bigend=bigend)

    def pcb_caplen(self):
        size = pad4bytes(self.caplen)
        self.vsGetField('data').vsSetLength(size)

    def setPcapTimestamp(self, idb):
        '''
        Adds a libpcap compatible tvsec and tvusec fields, based on the pcapng timestamp
        '''
        #orange left off here
        self.snaplen = idb.snaplen

        tstamp = (self.tstamphi << 32) | self.tstamplow
        scale = 1000000
        if idb.tsresol is None:
            #if not set, capture assumes 10e-6 resolution
            pass
        elif (0x80 & idb.tsresol) == 0:
            # remaining bits are resolution, to a negative power of 10
            scale = 10**(idb.tsresol & 0x7f)
        else:
            # remaining bits are resolution, to a negative power of 2
            scale = 1 << (idb.tsresol & 0x7f)

        self.tvsec = (tstamp / scale) + idb.tsoffset
        self.tvusec = tstamp % scale


class PCAPNG_SIMPLE_PACKET_BLOCK(vstruct.VStruct):
    '''
    Note: no variable length options fields, so inheriting from vstruct directly
    '''
    def __init__(self, bigend=False):
        vstruct.VStruct.__init__(self)
        self.blocktype      = v_uint32(bigend=bigend)
        self.blocksize      = v_uint32(bigend=bigend)
        self.packetlen      = v_uint32(bigend=bigend)
        self.data           = v_bytes(0)
        self.blocksize2     = v_uint32(bigend=bigend)

    def pcb_blocksize(self):
        self.caplen = pad4bytes(self.blocksize - 16)
        self.vsGetField('data').vsSetLength(self.caplen)

    def setPcapTimestamp(self, idb):
        #no timestamp in this type of block :(
        self.tvsec = idb.tsoffset
        self.tvusec = 0

def iterPcapFileName(filename, reuse=False):
    with open(filename, 'rb') as fd:
        for x in iterPcapFile(fd, reuse=reuse):
            yield x

def iterPcapFile(fd, reuse=False):
    '''
    Figure out if it's a tcpdump format, or pcapng
    '''
    h = PCAP_FILE_HEADER()
    b = fd.read(len(h))
    h.vsParse(b, fast=True)
    fd.seek(0)
    if h.magic == PCAPNG_BLOCKTYPE_SECTION_HEADER:
        return _iterPcapNgFile(fd, reuse)
    return _iterPcapFile(fd, reuse)

def _iterPcapFile(fd, reuse=False):

    h = PCAP_FILE_HEADER()
    b = fd.read(len(h))
    h.vsParse(b, fast=True)

    linktype = h.linktype

    if linktype not in (PCAP_LINKTYPE_ETHER, PCAP_LINKTYPE_RAW):
        raise Exception('PCAP Link Type %d Not Supported Yet!' % linktype)

    pkt = PCAP_PACKET_HEADER()
    eII = vs_inet.ETHERII()

    pktsize = len(pkt)
    eIIsize = len(eII)

    ipv4 = vs_inet.IPv4()
    ipv4size = 20
    tcp_hdr = vs_inet.TCP()
    udp_hdr = vs_inet.UDP()
    icmp_hdr = vs_inet.ICMP()

    go = True
    while go:

        hdr = fd.read(pktsize)
        if len(hdr) != pktsize:
            break

        pkt.vsParse(hdr, fast=True)

        b = fd.read(pkt.caplen)

        offset = 0

        if linktype == PCAP_LINKTYPE_ETHER:

            if len(b) < eIIsize:
                continue

            eII.vsParse(b, 0, fast=True)

            # No support for non-ip protocol yet...
            if eII.etype not in (vs_inet.ETH_P_IP,vs_inet.ETH_P_VLAN):
                continue

            offset += eIIsize

            if eII.etype == vs_inet.ETH_P_VLAN:
                offset +=  4

        elif linktype == PCAP_LINKTYPE_RAW:
            pass

        if not reuse:
            ipv4 = vs_inet.IPv4()

        if (len(b) - offset) < ipv4size:
            continue

        ipv4.vsParse(b, offset, fast=True)

        # Make b *only* the IP datagram bytes...
        b = b[offset:offset+ipv4.totlen]

        offset = 0
        offset += len(ipv4)
        tsize = len(b) - offset

        if ipv4.proto == vs_inet.IPPROTO_TCP:

            if tsize < 20:
                continue

            if not reuse:
                tcp_hdr = vs_inet.TCP()

            tcp_hdr.vsParse(b, offset, fast=True)
            offset += len(tcp_hdr)
            pdata = b[offset:]

            yield pkt,ipv4,tcp_hdr,pdata

        elif ipv4.proto == vs_inet.IPPROTO_UDP:

            if tsize < 8:
                continue

            if not reuse:
                udp_hdr = vs_inet.UDP()

            udp_hdr.vsParse(b, offset, fast=True)
            offset += len(udp_hdr)
            pdata = b[offset:]

            yield pkt,ipv4,udp_hdr,pdata

        elif ipv4.proto == vs_inet.IPPROTO_ICMP:
            if tsize < 4:
                continue

            if not reuse:
                icmp_hdr = vs_inet.ICMP()

            icmp_hdr.vsParse(b, offset, fast=True)
            offset += len(icmp_hdr)
            pdata = b[offset:]

            yield pkt,ipv4,icmp_hdr,pdata

        else:
            logger.warning('UNHANDLED IP PROTOCOL: %d', ipv4.proto)


def _iterPcapNgFile(fd, reuse=False):
    header = PCAPNG_GENERIC_BLOCK_HEADER()
    ifaceidx = 0
    ifacedict = {}
    roff = 0
    bigend = False
    curroff = fd.tell()
    b0 = fd.read(len(header))
    fd.seek(curroff)
    while len(b0) == len(header):
        header.vsParse(b0, fast=True)
        body = fd.read(header.blocksize)
        if header.blocktype == PCAPNG_BLOCKTYPE_SECTION_HEADER:
            shb = PCAPNG_SECTION_HEADER_BLOCK()
            roff = shb.vsParse(body)
            bigend = shb.bigend
            #reset interface stuff since we're in a new section
            ifaceidx = 0
            ifacedict = {}
        elif header.blocktype == PCAPNG_BLOCKTYPE_INTERFACE_DESCRIPTION:
            idb = PCAPNG_INTERFACE_DESCRIPTION_BLOCK(bigend)
            roff = idb.vsParse(body)
            #save off the interface for later reference
            ifacedict[ifaceidx] = idb
            ifaceidx += 1
        elif header.blocktype == PCAPNG_BLOCKTYPE_SIMPLE_PACKET:
            spb = PCAPNG_SIMPLE_PACKET_BLOCK(bigend)
            roff = spb.vsParse(body)
            tup = _parsePcapngPacketBytes(iface.linktype, spb)
            if tup is not None:
                #if it is None, just fall through & read next block
                yield tup
        elif header.blocktype == PCAPNG_BLOCKTYPE_ENHANCED_PACKET:
            epb = PCAPNG_ENHANCED_PACKET_BLOCK(bigend)
            roff = epb.vsParse(body)
            iface = ifacedict.get(epb.interfaceid)
            epb.setPcapTimestamp(iface)
            tup = _parsePcapngPacketBytes(iface.linktype, epb)
            if tup is not None:
                #if tup is None, just fall through & read next block
                yield tup

            #TODO: other blocks needed?
            #PCAPNG_BLOCKTYPE_PACKET (obsolete)
            #PCAPNG_BLOCKTYPE_NAME_RESOLUTION:
            #PCAPNG_BLOCKTYPE_INTERFACE_STATS:
        else:
            logger.warning('Unknown block type: 0x%08x: 0x%08x 0x%08x bytes', roff, header.blocktype, header.blocksize)
        curroff = fd.tell()
        b0 = fd.read(len(header))
        fd.seek(curroff)

def _parsePcapngPacketBytes(linktype, pkt):
    '''
    pkt is either a parsed PCAPNG_SIMPLE_PACKET_BLOCK or PCAPNG_ENHANCED_PACKET_BLOCK
    On success Returns tuple (pcapng_pkt, ipv4_vstruct, transport_vstruc, pdata)
    Returns None if the packet can't be parsed
    '''
    if linktype not in (PCAP_LINKTYPE_ETHER, PCAP_LINKTYPE_RAW):
        raise Exception('PCAP Link Type %d Not Supported Yet!' % linktype)

    #pkt = PCAP_PACKET_HEADER()
    eII = vs_inet.ETHERII()
    eIIsize = len(eII)

    offset = 0
    if linktype == PCAP_LINKTYPE_ETHER:
        if len(pkt.data) < eIIsize:
            return None
        eII.vsParse(pkt.data, 0, fast=True)
        # No support for non-ip protocol yet...
        if eII.etype not in (vs_inet.ETH_P_IP,vs_inet.ETH_P_VLAN):
            return None
        offset += eIIsize
        if eII.etype == vs_inet.ETH_P_VLAN:
            offset +=  4
    elif linktype == PCAP_LINKTYPE_RAW:
        pass

    ipv4 = vs_inet.IPv4()
    if (len(pkt.data) - offset) < len(ipv4):
        return None
    ipv4.vsParse(pkt.data, offset, fast=True)

    # Make b *only* the IP datagram bytes...
    b = pkt.data[offset:offset+ipv4.totlen]

    offset = 0
    offset += len(ipv4)
    tsize = len(b) - offset

    if ipv4.proto == vs_inet.IPPROTO_TCP:
        if tsize < 20:
            return None
        tcp_hdr = vs_inet.TCP()
        tcp_hdr.vsParse(b, offset, fast=True)
        offset += len(tcp_hdr)
        pdata = b[offset:]
        return pkt,ipv4,tcp_hdr,pdata
    elif ipv4.proto == vs_inet.IPPROTO_UDP:
        if tsize < 8:
            return None
        udp_hdr = vs_inet.UDP()
        udp_hdr.vsParse(b, offset, fast=True)
        offset += len(udp_hdr)
        pdata = b[offset:]
        return pkt,ipv4,udp_hdr,pdata
    elif ipv4.proto == vs_inet.IPPROTO_ICMP:
        if tsize < 4:
            return None
        icmp_hdr = vs_inet.ICMP()
        icmp_hdr.vsParse(b, offset, fast=True)
        offset += len(icmp_hdr)
        pdata = b[offset:]
        return pkt,ipv4,icmp_hdr,pdata
    else:
        logger.warning('UNHANDLED IP PROTOCOL: %d', ipv4.proto)
    return None



'''
An API implementing a few caching utilities.
( use carefully! caches are dangerous ;) )
'''
from functools import wraps
from collections import deque

class FixedDepthCache:
    '''
    A fixed depth cache.
    '''
    def __init__(self, depth, misscb=None, finicb=None):
        self.cache = {}
        self.depth = depth
        self.cdeque = deque()

        self.misscb = misscb
        self.finicb = finicb

    def clear(self):
        '''
        Clear all the elements of the cache.
        '''
        self.cache.clear()
        self.cdeque.clear()

    def get(self, key, default=None):
        '''
        Retrieve the given key's value in the cache.
        '''
        if key not in self.cache and self.misscb:
            val = self.misscb(key)
            self.put(key, val)
            return val

        return self.cache.get(key, default)

    def put(self, key, value):
        '''
        Insert the given key/value pair into the cache.
        '''
        self.cache[key] = value
        self.cdeque.append(key)
        while len(self.cdeque) > self.depth:
            popkey = self.cdeque.popleft()
            popval = self.cache.pop(popkey, None)
            if self.finicb:
                self.finicb(popkey, popval)

    def pop(self):
        key = self.cdeque.popleft()
        val = self.cache.pop(key)
        return (key, val)

    def has(self, key):
        return key in self.cache

    def __len__(self):
        return len(self.cdeque)

def cachefunc(depth):
    cache = FixedDepthCache(depth)

    def wrapdef(f):

        @wraps(f)
        def funcdef(*args):
            if not cache.has( args ):
                ret = f(*args)
                cache.put(args,ret)
                return ret
            return cache.get(args)

        return funcdef

    return wrapdef



# Copyright (C) 2007 Invisigoth - See LICENSE file for details

import logging
import hashlib
import collections

import vtrace
import vtrace.notifiers as v_notifiers
import vtrace.rmi as v_rmi

import envi
import envi.const as e_const
import envi.archs.i386 as e_i386
import envi.archs.amd64 as e_amd64


logger = logging.getLogger(__name__)


class TraceManager:
    """
    A trace-manager is a utility class to extend from when you may be dealing
    with multiple tracer objects.  It allows for persistant mode settings and
    persistent metadata as well as bundling a DistributedNotifier.  You may also
    extend from this to get auto-magic remote stuff for your managed traces.
    """
    def __init__(self, trace=None):
        self.trace = trace
        self.dnotif = v_notifiers.DistributedNotifier()
        self.modes = {}  # See docs for trace modes
        self.metadata = {}  # Like traces, but persistant

    def manageTrace(self, trace):
        """
        Set all the modes/meta/notifiers in this trace for management
        by this TraceManager.
        """
        self.trace = trace
        if vtrace.remote:
            trace.registerNotifier(vtrace.NOTIFY_ALL, v_rmi.getCallbackProxy(trace, self.dnotif))
        else:
            trace.registerNotifier(vtrace.NOTIFY_ALL, self.dnotif)

        for name, val in self.modes.items():
            trace.setMode(name, val)

        for name, val in self.metadata.items():
            trace.setMeta(name, val)

    def unManageTrace(self, trace):
        """
        Untie this trace manager from the trace.
        """
        if vtrace.remote:
            trace.deregisterNotifier(vtrace.NOTIFY_ALL, v_rmi.getCallbackProxy(trace, self.dnotif))
        else:
            trace.deregisterNotifier(vtrace.NOTIFY_ALL, self.dnotif)

    def setMode(self, name, value):
        if self.trace is not None:
            self.trace.setMode(name, value)
        self.modes[name] = value

    def getMode(self, name, default=False):
        if self.trace is not None:
            return self.trace.getMode(name, default)
        return self.modes.get(name, default)

    def setMeta(self, name, value):
        if self.trace is not None:
            self.trace.setMeta(name, value)
        self.metadata[name] = value

    def getMeta(self, name, default=None):
        if self.trace is not None:
            return self.trace.getMeta(name, default)
        return self.metadata.get(name, default)

    def registerNotifier(self, event, notif):
        self.dnotif.registerNotifier(event, notif)

    def deregisterNotifier(self, event, notif):
        self.dnotif.deregisterNotifier(event, notif)

    def fireLocalNotifiers(self, event, trace):
        """
        Deliver a local event to the DistributedNotifier managing
        the traces. (used to locally bump notifiers)
        """
        self.dnotif.notify(event, trace)


def emuFromTrace(trace):
    '''
    Produce an envi emulator for this tracer object.
    '''
    arch = trace.getMeta('Architecture')
    plat = trace.getMeta('Platform')
    amod = envi.getArchModule(arch)
    emu = amod.getEmulator()
    bmaps = trace.getMeta('BadMaps')
    [emu.setMeta(key, val) for key, val in trace.metadata.items()]

    # could use {get,set}MemorySnap if trace inherited from MemoryObject
    for va, size, perms, fname in trace.getMemoryMaps():
        try:
            # So linux maps in a PROT_NONE page for efficient library sharing, so we have to take that into account
            if not perms & e_const.MM_READ:
                continue

            if plat == 'linux' and fname in bmaps:
                continue
            bytez = trace.readMemory(va, size)
            emu.addMemoryMap(va, perms, fname, bytez)
        except vtrace.PlatformException:
            logger.warning('failed to map: 0x{:x} (size: {:d}) into emu'.format(va, size))
            continue

    rsnap = trace.getRegisterContext().getRegisterSnap()
    emu.setRegisterSnap(rsnap)

    if plat == 'windows':
        psize = trace.getPointerSize()
        # capture PEB and TIB
        peb = trace.getMeta('PEB')
        if hasattr(trace, 'win32threads'):
            tebs = dict(trace.win32threads)
            emu.setMeta('TEBs', tebs)
        else:
            metatebs = trace.getMeta('TEBs')
            if metatebs:
                emu.setMeta('TEBs', tebs)

        emu.setMeta('PEB', peb)

        seginfo = trace.getThreads()[trace.getMeta('ThreadId')]
        if psize == 4:
            emu.setSegmentInfo(e_i386.SEG_FS, seginfo, 0xffffffff)
        elif psize == 8:
            emu.setSegmentInfo(e_amd64.SEG_GS, seginfo, 0xffffffffffff)

    return emu


def vwFromTrace(trace, storagename='binary_workspace_from_vsnap.viv', filefmt=None, collapse=True, strict=True):
    '''
    Produce an envi emulator for this tracer object.

    If filefmt is None, it will be auto-determined

    If collapse, join adjacent maps
    If strict, only join maps with the same permissions
    '''
    import vivisect
    vw = vivisect.VivWorkspace()
    arch = trace.getMeta('Architecture')
    plat = trace.getMeta('Platform')

    # determine file format (if not specified above)
    if filefmt is None:
        if 'win' in plat.lower():
            filefmt = 'pe'
            from vivisect.parsers.pe import archcalls
        else:
            filefmt = 'elf'
            from vivisect.parsers.elf import archcalls
    else:
        # TODO: what should this be really be?
        archcalls = {}

    vw.setMeta("Architecture", arch)
    vw.setMeta("Platform", plat)
    vw.setMeta('Format', filefmt)
    vw.setMeta('DefaultCall', archcalls.get(arch, 'unknown'))
    vw.setMeta('StorageName', storagename)

    if 'win' in plat.lower():
        ossep = '\\'
        exts = ('exe', 'dll')
    else:
        ossep = '/'
        exts = ('so',)

    # could use {get,set}MemorySnap if trace inherited from MemoryObject
    maps = []
    fnames = collections.defaultdict(int)
    filemeta = collections.defaultdict(hashlib.md5)
    bmaps = trace.getMeta('BadMaps')

    for va, size, perms, fname in trace.getMemoryMaps():
        # strip off unwanted parts
        trimfname = fname.split(ossep)[-1]
        stripfname = trimfname
        for ext in exts:
            if trimfname.endswith('.' + ext):
                stripfname = trimfname[:-(len(ext) + 1)]

        # add map to the workspace
        try:
            # So linux maps in a PROT_NONE page for efficient library sharing, so we have to take that into account
            if not perms & e_const.MM_READ:
                continue
            if plat == 'linux' and stripfname in bmaps:
                continue
            bytez = trace.readMemory(va, size)
            maps.append((va, perms, stripfname, bytez))

        except vtrace.PlatformException:
            logger.warning('failed to map: 0x{:x} (size: {:d}) into emu'.format(va, size))
            continue

    # filter maps
    if collapse:
        maps = collapseMemoryMaps(maps, strict=strict)

    # add maps
    for va, perms, fname, bytez in maps:
        count = fnames.get(fname, 0)
        fnames[fname] = count + 1

        vw.addMemoryMap(va, perms, fname, bytez)
        vw.addSegment(va, len(bytez), "%s_%d" % (fname, count), fname)
        filemeta[fname].update(bytez)

    # now actually add the files
    for fname, fbyts in filemeta.items():
        # find first va:
        for va, perms, mnm, btz in maps:
            if mnm == fname:
                break

        vw.addFile(fname, va, fbyts.hexdigest())

    # windows stuff
    if plat == 'windows':
        # capture PEB and TIB
        peb = trace.getMeta('PEB')
        if hasattr(trace, 'win32threads'):
            tebs = dict(trace.win32threads)
            vw.setMeta('TEBs', tebs)
        else:
            metatebs = trace.getMeta('TEBs')
            if metatebs:
                vw.setMeta('TEBs', metatebs)

        vw.setMeta('PEB', peb)

    return vw


def collapseMemoryMaps(oldmaps, strict=True):
    '''
    Sort through a list of memory maps and collapse any which abutt.
    If strict, only collapse if the permissions are the same.
    Otherwise, collapse them and or the permissions together.

    TODO: make all map bytes collapsed and make maps start at an offset?
        or poss
    '''
    # if we have no maps, skip the whole process
    if not len(oldmaps):
        return

    oldmaps.sort()

    # start off with the current map as the first oldmap, and add it
    newmaps = [oldmaps[0]]
    curva, curperms, curfname, curbytez = oldmaps[0]
    cursz = len(curbytez)
    curvamax = curva + cursz
    logger.debug("initial map: 0x%x, perms:%x, %r, %d-bytes, curvamax: 0x%x", curva, curperms, curfname, cursz, curvamax)

    for omidx in range(1, len(oldmaps)):
        ova, operms, ofname, obytez = oldmaps[omidx]
        omsz = len(obytez)
        ovamax = ova + omsz
        logger.debug("next map: 0x%x, perms:%x, %r, %d-bytes, curvamax: 0x%x", ova, operms, ofname, omsz, ovamax)
        if ova == curvamax and curfname == ofname and (not strict or curperms == operms):
            # collapse this into previous and update curvamax and curbytes if perms or not strict
            curvamax = ovamax
            newfname = None
            if len(curfname):
                if len(ofname) and ofname != curfname:
                    newfname = '%s + %s' % (curfname, ofname)
                else:
                    newfname = curfname
            else:
                newfname = ofname

            curbytez += obytez
            curfname = newfname
            newmaps[-1] = curva, curperms, newfname, curbytez
            logger.debug("collapsing: initial map: 0x%x, perms:%x, %r, %d-bytes, curvamax: 0x%x", \
                    curva, curperms, curfname, cursz, curvamax)

        else:
            #logger.debug("ova (0x%x) != curvamax (0x%x) or curperms (%r) != operms (%r)", ova, curvamax, curperms, operms)
            # add this map to newmaps and update cur*
            newmaps.append(oldmaps[omidx])
            curva, curperms, curfname, curbytez = oldmaps[omidx]
            cursz = len(curbytez)
            curvamax = curva + cursz

    return newmaps




# APIs for Windows 32-bit.  Each library has its own file of APIs, and
# they are collected into a single dictionary here.  The separate files
# are hard-coded into this module.  Each is expected to return
# a dictionary named "api_defs".
#
# Format:  rettype, retname, callconv, exactname, arglist(type, name)
#          where arglist type is one of ['int', 'void *']
#                arglist name is one of [None, 'funcptr', 'obj', 'ptr']


apitypes = {
    # NTDLL
    'DWORD':        'unsigned int',
    'HANDLE':       'DWORD',
    'LIBHANDLE':    'HANDLE',
    'HEAP':         'HANDLE',
}


api = {}

import vivisect.impapi.windows.ntdll_32 as mNt
api.update(mNt.api_defs)

import vivisect.impapi.windows.kernel_32 as mKer
api.update(mKer.api_defs)

import vivisect.impapi.windows.advapi_32 as mAdv
api.update(mAdv.api_defs)

import vivisect.impapi.windows.gdi_32 as mGdi
api.update(mGdi.api_defs)

import vivisect.impapi.windows.ole_32 as mOle
api.update(mOle.api_defs)

import vivisect.impapi.windows.user_32 as mUser
api.update(mUser.api_defs)

import vivisect.impapi.windows.rpcrt4_32 as mRpc
api.update(mRpc.api_defs)

import vivisect.impapi.windows.msvcrt_32 as mCrt
api.update(mCrt.api_defs)

import vivisect.impapi.windows.msvcr71_32 as mCr71
api.update(mCr71.api_defs)

import vivisect.impapi.windows.msvcr80_32 as mCr80
api.update(mCr80.api_defs)

import vivisect.impapi.windows.msvcr90_32 as mCr90
api.update(mCr90.api_defs)

import vivisect.impapi.windows.msvcr100_32 as mCr100
api.update(mCr100.api_defs)

import vivisect.impapi.windows.msvcr110_32 as mCr110
api.update(mCr110.api_defs)

import vivisect.impapi.windows.msvcr120_32 as mCr120
api.update(mCr120.api_defs)

import vivisect.impapi.windows.ws2plus_32 as mWs2
api.update(mWs2.api_defs)

import vivisect.impapi.windows.shell_32 as mShl
api.update(mShl.api_defs)
